@@ -34,6 +34,44 @@
 #![allow(unknown_lints)]
 #![allow(inline_always)]
 
+#![no_std]
+#![cfg_attr(not(feature = "std"), feature(alloc))]
+
+#[cfg(feature = "std")]
+#[macro_use]
+extern crate std;
+
+#[cfg(not(feature = "std"))]
+#[macro_use]
+extern crate core as std;
+
+#[cfg(not(feature = "std"))]
+#[macro_use]
+extern crate alloc;
+
+#[cfg(feature = "std")]
+mod imports {
+    pub use std::boxed;
+    pub use std::vec;
+    pub use std::string;
+    pub use std::borrow;
+    pub use std::collections::btree_map;
+}
+
+#[cfg(not(feature = "std"))]
+mod imports {
+    pub use alloc::boxed;
+    pub use alloc::vec;
+    pub use alloc::string;
+    pub use alloc::borrow;
+    pub use alloc::collections::btree_map;
+}
+
+use imports::*;
+
+use string::String;
+use vec::Vec;
+
 #[macro_use]
 mod logging;
 
@@ -46,7 +84,6 @@ use ast::{Demangle, Parse, ParseContext};
 use error::{Error, Result};
 use index_str::IndexStr;
 use std::fmt;
-use std::io;
 
 /// Options to control the demangling process.
 #[derive(Clone, Copy, Debug, Default)]
@@ -129,18 +166,16 @@ where
             parsed: parsed,
         };
 
-        if cfg!(feature = "logging") {
-            println!(
-                "Successfully parsed '{}' as
+        log!(
+            "Successfully parsed '{}' as
 
 AST = {:#?}
 
 substitutions = {:#?}",
-                String::from_utf8_lossy(symbol.raw.as_ref()),
-                symbol.parsed,
-                symbol.substitutions
-            );
-        }
+            String::from_utf8_lossy(symbol.raw.as_ref()),
+            symbol.parsed,
+            symbol.substitutions
+        );
 
         Ok(symbol)
     }
@@ -164,8 +199,8 @@ substitutions = {:#?}",
     /// let demangled_again = sym.demangle(&options).unwrap();
     /// assert_eq!(demangled_again, demangled);
     /// ```
-    pub fn demangle(&self, options: &DemangleOptions) -> io::Result<String> {
-        let mut out = vec![];
+    pub fn demangle(&self, options: &DemangleOptions) -> ::std::result::Result<String, fmt::Error> {
+        let mut out = String::new();
         {
             let mut ctx = ast::DemangleContext::new(
                 &self.substitutions,
@@ -176,7 +211,7 @@ substitutions = {:#?}",
             self.parsed.demangle(&mut ctx, None)?;
         }
 
-        Ok(String::from_utf8(out).unwrap())
+        Ok(out)
     }
 }
 
@@ -216,18 +251,16 @@ impl<T> Symbol<T> {
             parsed: parsed,
         };
 
-        if cfg!(feature = "logging") {
-            println!(
-                "Successfully parsed '{}' as
+        log!(
+            "Successfully parsed '{}' as
 
 AST = {:#?}
 
 substitutions = {:#?}",
-                String::from_utf8_lossy(symbol.raw.as_ref()),
-                symbol.parsed,
-                symbol.substitutions
-            );
-        }
+            String::from_utf8_lossy(symbol.raw.as_ref()),
+            symbol.parsed,
+            symbol.substitutions
+        );
 
         Ok((symbol, tail.into()))
     }
@@ -238,7 +271,7 @@ where
     T: AsRef<[u8]>,
 {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let mut out = vec![];
+        let mut out = String::new();
         {
             let options = DemangleOptions::default();
             let mut ctx = ast::DemangleContext::new(
@@ -252,6 +285,6 @@ where
                 fmt::Error
             })?;
         }
-        write!(f, "{}", String::from_utf8_lossy(&out))
+        write!(f, "{}", &out)
     }
 }
